@@ -3,11 +3,12 @@ from typing import Dict, List, Optional, Type
 import jwt
 from fastapi import FastAPI, HTTPException
 from starlette.status import HTTP_403_FORBIDDEN
-from tortoise import Model, Tortoise
+from tortoise import Model
 
-from .common import import_obj, pwd_context
+from . import enums
+from .common import get_all_models, import_obj, pwd_context
 from .exceptions import exception_handler
-from .models import AbstractPermission, AbstractRole, AbstractUser
+from .models import AbstractAdminLog, AbstractPermission, AbstractRole, AbstractUser
 from .schemas import LoginIn
 from .shortcuts import get_object_or_404
 from .site import Field, Menu, Resource, Site
@@ -35,8 +36,12 @@ class AdminApp(FastAPI):
     models: Dict[str, Type[Model]] = {}
     admin_secret: str
     user_model: Type[Model]
+    permission_model: Type[Model]
+    role_model: Type[Model]
+    admin_log_model: Type[Model]
     site: Site
     permission: bool
+    admin_log: bool
     _inited: bool = False
     _field_type_mapping = {
         "IntField": "number",
@@ -76,7 +81,7 @@ class AdminApp(FastAPI):
 
     def _build_content_menus(self) -> List[Menu]:
         menus = []
-        for model_name, model in self._get_all_models():
+        for model_name, model in get_all_models():
             if issubclass(model, (AbstractUser, AbstractPermission, AbstractRole)):
                 continue
             menu = Menu(
@@ -97,71 +102,91 @@ class AdminApp(FastAPI):
 
         menus = [
             Menu(name="Home", url="/", icon="fa fa-home"),
-            Menu(name="Content", title=True),
-            *self._build_content_menus(),
-            Menu(name="External", title=True),
+            Menu(name="Content", children=self._build_content_menus()),
             Menu(
-                name="Github",
-                url="https://github.com/long2ice/fastapi-admin",
-                icon="fa fa-github",
-                external=True,
+                name="External",
+                children=[
+                    Menu(
+                        name="Github",
+                        url="https://github.com/long2ice/fastapi-admin",
+                        icon="fa fa-github",
+                        external=True,
+                    ),
+                ],
             ),
         ]
         if permission:
             permission_menus = [
-                Menu(name="Auth", title=True),
                 Menu(
-                    name="User", url="/rest/User", icon="fa fa-user", search_fields=("username",),
+                    name="Auth",
+                    children=[
+                        Menu(
+                            name="User",
+                            url="/rest/User",
+                            icon="fa fa-user",
+                            search_fields=("username",),
+                        ),
+                        Menu(name="Role", url="/rest/Role", icon="fa fa-group",),
+                        Menu(name="Permission", url="/rest/Permission", icon="fa fa-user-plus",),
+                        Menu(name="Logout", url="/logout", icon="fa fa-lock",),
+                    ],
                 ),
-                Menu(name="Role", url="/rest/Role", icon="fa fa-group", actions={"delete": False}),
-                Menu(
-                    name="Permission",
-                    url="/rest/Permission",
-                    icon="fa fa-user-plus",
-                    actions={"delete": False},
-                ),
-                Menu(name="Logout", url="/logout", icon="fa fa-lock",),
             ]
             menus += permission_menus
         return menus
 
-    def init(
+    async def init(
         self,
         site: Site,
         admin_secret: str,
         permission: bool = False,
+        admin_log: bool = False,
         login_view: Optional[str] = None,
     ):
         """
         init admin site
+        :param admin_log:
         :param login_view:
-        :param tortoise_app:
         :param permission: active builtin permission
         :param site:
-        :param user_model: admin user model path,like admin.models.user
         :param admin_secret: admin jwt secret.
         :return:
         """
         self.site = site
         self.permission = permission
         self.admin_secret = admin_secret
-        for model_name, model in self._get_all_models():
+        self.admin_log = admin_log
+        for model_name, model in get_all_models():
             if issubclass(model, AbstractUser):
                 self.user_model = model
+            elif issubclass(model, AbstractAdminLog):
+                self.admin_log_model = model
             self.models[model_name] = model
         self._inited = True
         if not site.menus:
             site.menus = self._build_default_menus(permission)
+        if permission:
+            await self._register_permissions()
         self._get_model_menu_mapping(site.menus)
         if login_view:
             self.add_api_route("/login", import_obj(login_view), methods=["POST"])
         else:
             self.add_api_route("/login", login, methods=["POST"])
 
-    def _get_all_models(self):
-        for tortoise_app, models in Tortoise.apps.items():
-            for model_item in models.items():
-                yield model_item
+    async def _register_permissions(self):
+        permission_model = None
+        for model_name, model in get_all_models():
+            if issubclass(model, AbstractPermission):
+                permission_model = model
+                break
+        if not permission_model:
+            raise Exception("No Permission Model Founded.")
+
+        for model, _ in get_all_models():
+            for action in enums.PermissionAction.choices():
+                label = f"{enums.PermissionAction.choices().get(action)} {model}"
+                defaults = dict(label=label, model=model, action=action,)
+                await permission_model.get_or_create(**defaults,)
 
     def _exclude_field(self, resource: str, field: str):
         """
@@ -255,6 +280,8 @@ class AdminApp(FastAPI):
                 disabled=readonly,
                 description=data_field.get("description"),
             )
+            if field_type == "DecimalField" or field_type == "FloatField":
+                field.step = "any"
             field = field.copy(update=menu.attrs.get(name) or {})
             fields[name] = field
             if name in search_fields:
@@ -317,6 +344,7 @@ class AdminApp(FastAPI):
             pk=pk,
             bulk_actions=menu.bulk_actions,
             export=menu.export,
+            import_=menu.import_,
         )
 
 
